module.exports = {
    displayName: "mobile",
    resolver: "@nrwl/jest/plugins/resolver",
    preset: "jest-expo",
    transformIgnorePatterns: [
        "node_modules/(?!((jest-)?react-native|@react-native(-community)?)|expo(nent)?|@expo(nent)?/.*|@expo-google-fonts/.*|react-navigation|@react-navigation/.*|@unimodules/.*|unimodules|sentry-expo|native-base|react-native-svg)",
    ],
    moduleFileExtensions: ["ts", "js", "html", "tsx", "jsx"],
    setupFilesAfterEnv: ["<rootDir>/test-setup.ts"],
    moduleNameMapper: {
        ".svg": "@nrwl/expo/plugins/jest/svg-mock",
    },
    transform: {
        "\\.(js|ts|tsx)$": require.resolve("react-native/jest/preprocessor.js"),
        "^.+\\.(bmp|gif|jpg|jpeg|mp4|png|psd|svg|webp|ttf)$": require.resolve(
            "react-native/jest/assetFileTransformer.js"
        ),
    },
};
