import { Calculate, CompareArrows, Person, ShoppingCart } from "@mui/icons-material";

export const ClearingAccountIcon = Calculate;
export const PersonalAccountIcon = Person;
export const TransferIcon = CompareArrows;
export const PurchaseIcon = ShoppingCart;
